@@ -1,6 +1,6 @@
 extern crate cpython;
 
-use cpython::{PythonObject, Python};
+use cpython::Python;
 use cpython::ObjectProtocol; //for call method
 
 fn main() {
@@ -8,11 +8,11 @@ fn main() {
     let py = gil.python();
 
     let sys = py.import("sys").unwrap();
-    let version: String = sys.get("version").unwrap().extract().unwrap();
+    let version: String = sys.get(py, "version").unwrap().extract(py).unwrap();
 
     let os = py.import("os").unwrap();
-    let getenv = os.get("getenv").unwrap();
-    let user: String = getenv.call(("USER",), None).unwrap().extract().unwrap();
+    let getenv = os.get(py, "getenv").unwrap();
+    let user: String = getenv.call(py, ("USER",), None).unwrap().extract(py).unwrap();
 
     println!("Hello {}, I'm Python {}", user, version);
 }
