@@ -17,12 +17,15 @@
 // DEALINGS IN THE SOFTWARE.
 
 #![feature(unsafe_no_drop_flag)] // crucial so that PyObject<'p> is binary compatible with *mut ffi::PyObject
-#![feature(filling_drop)] // necessary to avoid segfault with unsafe_no_drop_flag
-#![feature(optin_builtin_traits)] // for opting out of Sync/Send
-#![feature(slice_patterns)] // for tuple_conversion macros
-#![feature(utf8_error)] // for translating Utf8Error to Python exception
-#![feature(plugin)]
-#![plugin(interpolate_idents)]
+#![feature(filling_drop)] // necessary to avoid segfault with unsafe_no_drop_flag (#5016)
+#![feature(optin_builtin_traits)] // for opting out of Sync/Send (#13231)
+#![feature(stmt_expr_attributes)] // easier python 2.x/3.x distinction (#15701)
+#![feature(const_fn)] // for GILProtected::new (#24111)
+#![feature(shared)] // for std::ptr::Shared (#27730)
+
+#![feature(plugin)]            // necessary because `fn concat_idents!(...)()` is 
+#![plugin(interpolate_idents)] // not supported by the current macro system.
+
 #![allow(unused_imports)] // because some imports are only necessary with python 2.x or 3.x
 
 //! Rust bindings to the Python interpreter.
@@ -31,28 +34,25 @@
 //! In Python, all objects are implicitly reference counted.
 //! In rust, we will use the `PyObject` type to represent a reference to a Python object.
 //!
-//! Because all Python objects potentially have multiple owners, the concept
-//! concept of rust mutability does not apply to Python objects.
+//! The method `clone_ref()` (from trait `PyClone`) can be used to create additional
+//! references to the same Python object.
+//!
+//! Because all Python objects potentially have multiple owners, the
+//! concept of Rust mutability does not apply to Python objects.
 //! As a result, this API will allow mutating Python objects even if they are not stored
-//! in a mutable rust variable.
+//! in a mutable Rust variable.
 //!
 //! The Python interpreter uses a global interpreter lock (GIL)
 //! to ensure thread-safety.
-//! This API uses the lifetime parameter `PyObject<'p>` to ensure that Python objects cannot
-//! be accessed without holding the GIL.
-//! Throughout this library, the lifetime `'p` always refers to the lifetime of the Python interpreter.
+//! This API uses a zero-sized `struct Python<'p>` as a token to indicate
+//! that a function can assume that the GIL is held.
 //!
-//! When accessing existing objects, the lifetime on `PyObject<'p>` is sufficient to ensure that the GIL
-//! is held by the current code. But we also need to ensure that the GIL is held when creating new objects.
-//! For this purpose, this library uses the marker type `Python<'p>`,
-//! which acts like a reference to the whole Python interpreter.
-//!
-//! You can obtain a `Python<'p>` instance by acquiring the GIL, or by calling `Python()`
-//! on any existing Python object.
+//! You obtain a `Python` instance by acquiring the GIL,
+//! and have to pass it into all operations that call into the Python runtime.
 //!
 //! # Error Handling
-//! The vast majority of operations in this library will return `PyResult<'p, ...>`.
-//! This is an alias for the type `Result<..., PyErr<'p>>`.
+//! The vast majority of operations in this library will return `PyResult<...>`.
+//! This is an alias for the type `Result<..., PyErr>`.
 //!
 //! A `PyErr` represents a Python exception. Errors within the rust-cpython library are
 //! also exposed as Python exceptions.
@@ -66,14 +66,14 @@
 //!
 //! fn main() {
 //!     let gil = Python::acquire_gil();
-//!     let py = gil.python();
+//!     let py = gil.python(); // obtain `Python` token
 //!
 //!     let sys = py.import("sys").unwrap();
-//!     let version: String = sys.get("version").unwrap().extract().unwrap();
+//!     let version: String = sys.get(py, "version").unwrap().extract(py).unwrap();
 //!
 //!     let os = py.import("os").unwrap();
-//!     let getenv = os.get("getenv").unwrap();
-//!     let user: String = getenv.call(("USER",), None).unwrap().extract().unwrap();
+//!     let getenv = os.get(py, "getenv").unwrap();
+//!     let user: String = getenv.call(py, ("USER",), None).unwrap().extract(py).unwrap();
 //!
 //!     println!("Hello {}, I'm Python {}", user, version);
 //! }
@@ -93,7 +93,7 @@ extern crate python3_sys as ffi;
 pub use ffi::Py_ssize_t;
 pub use err::{PyErr, PyResult};
 pub use objects::*;
-pub use python::{Python, PythonObject, PythonObjectWithCheckedDowncast, PythonObjectWithTypeObject};
+pub use python::{Python, PythonObject, PythonObjectWithCheckedDowncast, PythonObjectWithTypeObject, PyClone, PyDrop};
 pub use pythonrun::{GILGuard, GILProtected, prepare_freethreaded_python};
 pub use conversion::{ExtractPyObject, ToPyObject};
 pub use objectprotocol::{ObjectProtocol};
@@ -133,16 +133,20 @@ mod rustobject;
 /// Private re-exports for macros. Do not use.
 #[doc(hidden)]
 pub mod _detail {
-    pub use ffi;
-    pub use libc;
+    pub mod ffi {
+        pub use ::ffi::*;
+    }
+    pub mod libc {
+        pub use ::libc::c_char;
+    }
     pub use abort_on_panic::PanicGuard;
     pub use err::from_owned_ptr_or_panic;
-    pub use function::py_fn_impl;
+    pub use function::{get_kwargs, result_to_ptr, py_fn_impl};
     pub use rustobject::method::{py_method_impl, py_class_method_impl};
 
     /// assume_gil_acquired(), but the returned Python<'p> is bounded by the scope
     /// of the referenced variable.
-    /// This is useful in macros to ensure that type inference doesn't set 'p == 'static.
+    /// This is useful in macros to ensure that type inference doesn't set `'p` == `'static`.
     #[inline]
     pub unsafe fn bounded_assume_gil_acquired<'p, T>(_bound: &'p T) -> super::Python<'p> {
         super::Python::assume_gil_acquired()
@@ -154,12 +158,8 @@ pub mod _detail {
 ///
 /// Macro syntax: `py_module_initializer!($name, |$py, $m| $body)`
 ///
-/// 1. The module name as a string literal.
-/// 2. The name of the init function as an identifier.
-///    The function must be named `init$module_name` so that Python 2.7 can load the module.
-///    Note: this parameter will be removed in a future version
-///    (once Rust supports `concat_ident!` as function name).
-/// 3. A function or lambda of type `Fn(Python<'p>, &PyModule<'p>) -> PyResult<'p, ()>`.
+/// 1. `name`: The module name as a Rust identifier.
+/// 2. A lambda of type `Fn(Python, &PyModule) -> PyResult<()>`.
 ///    This function will be called when the module is imported, and is responsible
 ///    for adding the module's members.
 ///
@@ -172,12 +172,12 @@ pub mod _detail {
 /// use cpython::{Python, PyResult, PyObject};
 ///
 /// py_module_initializer!(example, |py, m| {
-///     try!(m.add("__doc__", "Module documentation string"));
-///     try!(m.add("run", py_fn!(run())));
+///     try!(m.add(py, "__doc__", "Module documentation string"));
+///     try!(m.add(py, "run", py_fn!(run())));
 ///     Ok(())
 /// });
 ///
-/// fn run<'p>(py: Python<'p>) -> PyResult<'p, PyObject<'p>> {
+/// fn run(py: Python) -> PyResult<PyObject> {
 ///     println!("Rust says: Hello Python!");
 ///     Ok(py.None())
 /// }
@@ -204,7 +204,7 @@ macro_rules! py_module_initializer {
         #[allow(non_snake_case)]
         pub unsafe extern "C" fn [ init $name ]() {
             // Nest init function so that $body isn't in unsafe context
-            fn init<'pmisip>($py_id: $crate::Python<'pmisip>, $m_id: &$crate::PyModule<'pmisip>) -> $crate::PyResult<'pmisip, ()> {
+            fn init($py_id: $crate::Python, $m_id: &$crate::PyModule) -> $crate::PyResult<()> {
                 $body
             }
             let name = concat!(stringify!($name), "\0").as_ptr() as *const _;
@@ -218,7 +218,7 @@ macro_rules! py_module_initializer {
 #[cfg(feature="python27-sys")]
 pub unsafe fn py_module_initializer_impl(
     name: *const libc::c_char,
-    init: for<'p> fn(Python<'p>, &PyModule<'p>) -> PyResult<'p, ()>
+    init: fn(Python, &PyModule) -> PyResult<()>
 ) {
     abort_on_panic!({
         let py = Python::assume_gil_acquired();
@@ -226,16 +226,16 @@ pub unsafe fn py_module_initializer_impl(
         let module = ffi::Py_InitModule(name, ptr::null_mut());
         if module.is_null() { return; }
 
-        let module = match PyObject::from_borrowed_ptr(py, module).cast_into::<PyModule>() {
+        let module = match PyObject::from_borrowed_ptr(py, module).cast_into::<PyModule>(py) {
             Ok(m) => m,
             Err(e) => {
-                PyErr::from(e).restore();
+                PyErr::from(e).restore(py);
                 return;
             }
         };
         match init(py, &module) {
             Ok(()) => (),
-            Err(e) => e.restore()
+            Err(e) => e.restore(py)
         }
     })
 }
@@ -248,7 +248,7 @@ macro_rules! py_module_initializer {
         #[allow(non_snake_case)]
         pub unsafe extern "C" fn [ PyInit_ $name ]() -> *mut $crate::_detail::ffi::PyObject {
             // Nest init function so that $body isn't in unsafe context
-            fn init<'pmisip>($py_id: $crate::Python<'pmisip>, $m_id: &$crate::PyModule<'pmisip>) -> $crate::PyResult<'pmisip, ()> {
+            fn init($py_id: $crate::Python, $m_id: &$crate::PyModule) -> $crate::PyResult<()> {
                 $body
             }
             static mut module_def: $crate::_detail::ffi::PyModuleDef = $crate::_detail::ffi::PyModuleDef {
@@ -275,7 +275,7 @@ macro_rules! py_module_initializer {
 #[cfg(feature="python3-sys")]
 pub unsafe fn py_module_initializer_impl(
     def: *mut ffi::PyModuleDef,
-    init: for<'p> fn(Python<'p>, &PyModule<'p>) -> PyResult<'p, ()>
+    init: fn(Python, &PyModule) -> PyResult<()>
 ) -> *mut ffi::PyObject {
     abort_on_panic!({
         let py = Python::assume_gil_acquired();
@@ -283,20 +283,19 @@ pub unsafe fn py_module_initializer_impl(
         let module = ffi::PyModule_Create(def);
         if module.is_null() { return module; }
 
-        let module = match PyObject::from_owned_ptr(py, module).cast_into::<PyModule>() {
+        let module = match PyObject::from_owned_ptr(py, module).cast_into::<PyModule>(py) {
             Ok(m) => m,
             Err(e) => {
-                PyErr::from(e).restore();
+                PyErr::from(e).restore(py);
                 return ptr::null_mut();
             }
         };
         match init(py, &module) {
             Ok(()) => module.into_object().steal_ptr(),
             Err(e) => {
-                e.restore();
+                e.restore(py);
                 return ptr::null_mut();
             }
         }
     })
 }
-
