@@ -19,7 +19,7 @@
 use std;
 use ffi;
 use libc::c_char;
-use python::{Python, PythonObject, ToPythonPointer};
+use python::{Python, PythonObject};
 use objectprotocol::ObjectProtocol;
 use conversion::ToPyObject;
 use objects::{PyObject, PyTuple, PyDict, exc};
@@ -27,13 +27,13 @@ use err::{self, PyResult, PyErr};
 use std::ffi::{CStr, CString};
 
 /// Represents a Python module object.
-pub struct PyModule<'p>(PyObject<'p>);
+pub struct PyModule(PyObject);
 
 pyobject_newtype!(PyModule, PyModule_Check, PyModule_Type);
 
-impl <'p> PyModule<'p> {
+impl PyModule {
     /// Create a new module object with the `__name__` attribute set to name.
-    pub fn new(py: Python<'p>, name: &str) -> PyResult<'p, PyModule<'p>> {
+    pub fn new(py: Python, name: &str) -> PyResult<PyModule> {
         let name = CString::new(name).unwrap();
         unsafe {
             err::result_cast_from_owned_ptr(py, ffi::PyModule_New(name.as_ptr()))
@@ -41,7 +41,7 @@ impl <'p> PyModule<'p> {
     }
 
     /// Import the Python module with the specified name.
-    pub fn import(py: Python<'p>, name: &str) -> PyResult<'p, PyModule<'p>> {
+    pub fn import(py: Python, name: &str) -> PyResult<PyModule> {
         let name = CString::new(name).unwrap();
         unsafe {
             err::result_cast_from_owned_ptr(py, ffi::PyImport_ImportModule(name.as_ptr()))
@@ -50,23 +50,21 @@ impl <'p> PyModule<'p> {
 
     /// Return the dictionary object that implements module's namespace;
     /// this object is the same as the `__dict__` attribute of the module object.
-    pub fn dict(&self) -> PyDict<'p> {
-        let py = self.python();
+    pub fn dict(&self, py: Python) -> PyDict {
         unsafe {
-            let r = PyObject::from_borrowed_ptr(py, ffi::PyModule_GetDict(self.as_ptr()));
+            let r = PyObject::from_borrowed_ptr(py, ffi::PyModule_GetDict(self.0.as_ptr()));
             r.unchecked_cast_into::<PyDict>()
         }
     }
 
-    unsafe fn str_from_ptr<'a>(&'a self, ptr: *const c_char) -> PyResult<'p, &'a str> {
-        let py = self.python();
-        if ptr == std::ptr::null() {
+    unsafe fn str_from_ptr<'a>(&'a self, py: Python, ptr: *const c_char) -> PyResult<&'a str> {
+        if ptr.is_null() {
             Err(PyErr::fetch(py))
         } else {
             let slice = CStr::from_ptr(ptr).to_bytes();
             match std::str::from_utf8(slice) {
                 Ok(s) => Ok(s),
-                Err(e) => Err(PyErr::from_instance(try!(exc::UnicodeDecodeError::new_utf8(py, slice, e))))
+                Err(e) => Err(PyErr::from_instance(py, try!(exc::UnicodeDecodeError::new_utf8(py, slice, e))))
             }
         }
     }
@@ -74,35 +72,36 @@ impl <'p> PyModule<'p> {
     /// Gets the module name.
     ///
     /// May fail if the module does not have a `__name__` attribute.
-    pub fn name<'a>(&'a self) -> PyResult<'p, &'a str> {
-        unsafe { self.str_from_ptr(ffi::PyModule_GetName(self.as_ptr())) }
+    pub fn name<'a>(&'a self, py: Python) -> PyResult<&'a str> {
+        unsafe { self.str_from_ptr(py, ffi::PyModule_GetName(self.0.as_ptr())) }
     }
 
     /// Gets the module filename.
     ///
     /// May fail if the module does not have a `__file__` attribute.
-    pub fn filename<'a>(&'a self) -> PyResult<'p, &'a str> {
-        unsafe { self.str_from_ptr(ffi::PyModule_GetFilename(self.as_ptr())) }
+    pub fn filename<'a>(&'a self, py: Python) -> PyResult<&'a str> {
+        unsafe { self.str_from_ptr(py, ffi::PyModule_GetFilename(self.0.as_ptr())) }
     }
 
     /// Gets a member from the module.
     /// This is equivalent to the Python expression: `getattr(module, name)`
-    pub fn get(&self, name: &str) -> PyResult<'p, PyObject<'p>> {
-        self.as_object().getattr(name)
+    pub fn get(&self, py: Python, name: &str) -> PyResult<PyObject> {
+        self.as_object().getattr(py, name)
     }
 
     /// Calls a function in the module.
     /// This is equivalent to the Python expression: `getattr(module, name)(*args, **kwargs)`
-    pub fn call<A>(&self, name: &str, args: A, kwargs: Option<&PyDict<'p>>) -> PyResult<'p, PyObject<'p>>
-      where A: ToPyObject<'p, ObjectType=PyTuple<'p>> {
-        try!(self.as_object().getattr(name)).call(args, kwargs)
+    pub fn call<A>(&self, py: Python, name: &str, args: A, kwargs: Option<&PyDict>) -> PyResult<PyObject>
+        where A: ToPyObject<ObjectType=PyTuple>
+    {
+        try!(self.as_object().getattr(py, name)).call(py, args, kwargs)
     }
 
     /// Adds a member to the module.
     ///
     /// This is a convenience function which can be used from the module's initialization function.
-    pub fn add<V>(&self, name: &str, value: V) -> PyResult<'p, ()> where V: ToPyObject<'p> {
-        self.dict().set_item(name, value)
+    pub fn add<V>(&self, py: Python, name: &str, value: V) -> PyResult<()> where V: ToPyObject {
+        self.as_object().setattr(py, name, value)
     }
 
     /// Adds a new extension type to the module.
@@ -110,9 +109,9 @@ impl <'p> PyModule<'p> {
     /// This is a convenience function that creates a new `PyRustTypeBuilder` and
     /// sets `new_type.__module__` to this module's name.
     /// The new type will be added to this module when `finish()` is called on the builder.
-    pub fn add_type<T>(&self, name: &str) -> ::rustobject::typebuilder::PyRustTypeBuilder<'p, T>
-            where T: 'p + Send {
-        ::rustobject::typebuilder::new_typebuilder_for_module(self, name)
+    pub fn add_type<'p, T>(&self, py: Python<'p>, name: &str) -> ::rustobject::typebuilder::PyRustTypeBuilder<'p, T>
+            where T: 'static + Send {
+        ::rustobject::typebuilder::new_typebuilder_for_module(py, self, name)
     }
 }
 
