@@ -17,33 +17,34 @@
 // DEALINGS IN THE SOFTWARE.
 
 use python::{Python, PythonObject, ToPythonPointer};
-use err::{self, PyResult, PyErr};
+use err::{self, PyErr, PyResult};
 use super::object::PyObject;
 use super::exc;
 use ffi::{self, Py_ssize_t};
 use conversion::{ToPyObject, ExtractPyObject};
+use std::slice;
 
 /// Represents a Python tuple object.
-pub struct PyTuple<'p>(PyObject<'p>);
+pub struct PyTuple(PyObject);
 
 pyobject_newtype!(PyTuple, PyTuple_Check, PyTuple_Type);
 
-impl <'p> PyTuple<'p> {
+impl PyTuple {
     /// Construct a new tuple with the given elements.
-    pub fn new(py: Python<'p>, elements: &[PyObject<'p>]) -> PyTuple<'p> {
+    pub fn new(py: Python, elements: &[PyObject]) -> PyTuple {
         unsafe {
             let len = elements.len();
             let ptr = ffi::PyTuple_New(len as Py_ssize_t);
             let t = err::result_cast_from_owned_ptr::<PyTuple>(py, ptr).unwrap();
             for (i, e) in elements.iter().enumerate() {
-                ffi::PyTuple_SetItem(ptr, i as Py_ssize_t, e.clone().steal_ptr());
+                ffi::PyTuple_SetItem(ptr, i as Py_ssize_t, e.steal_ptr(py));
             }
             t
         }
     }
 
     /// Retrieves the empty tuple.
-    pub fn empty(py: Python<'p>) -> PyTuple<'p> {
+    pub fn empty(py: Python) -> PyTuple {
         unsafe {
             err::result_cast_from_owned_ptr::<PyTuple>(py, ffi::PyTuple_New(0)).unwrap()
         }
@@ -51,74 +52,85 @@ impl <'p> PyTuple<'p> {
 
     /// Gets the length of the tuple.
     #[inline]
-    pub fn len(&self) -> usize {
-        // non-negative Py_ssize_t should always fit into Rust uint
+    pub fn len(&self, _py: Python) -> usize {
         unsafe {
-            ffi::PyTuple_Size(self.as_ptr()) as usize
+            // non-negative Py_ssize_t should always fit into Rust uint
+            ffi::PyTuple_GET_SIZE(self.0.as_ptr()) as usize
         }
     }
 
     /// Gets the item at the specified index.
     ///
     /// Panics if the index is out of range.
-    pub fn get_item(&self, index: usize) -> PyObject<'p> {
-        assert!(index < self.len());
+    pub fn get_item(&self, py: Python, index: usize) -> PyObject {
+        // TODO: reconsider whether we should panic
+        // It's quite inconsistent that this method takes `Python` when `len()` does not.
+        assert!(index < self.len(py));
         unsafe {
-            PyObject::from_borrowed_ptr(self.python(), ffi::PyTuple_GetItem(self.as_ptr(), index as Py_ssize_t))
+            PyObject::from_borrowed_ptr(py, ffi::PyTuple_GET_ITEM(self.0.as_ptr(), index as Py_ssize_t))
         }
     }
 
-    /* Disabled for now; we might want to change the PyObject memory layout for
-       compatiblity with stable Rust.
     #[inline]
-    pub fn as_slice<'a>(&'a self) -> &'a [PyObject<'p>] {
+    pub fn as_slice<'a>(&'a self) -> &'a [PyObject] {
         // This is safe because PyObject has the same memory layout as *mut ffi::PyObject,
         // and because tuples are immutable.
+        // (We don't even need a Python token, thanks to immutability)
         unsafe {
-            let ptr = self.as_ptr() as *mut ffi::PyTupleObject;
-            std::mem::transmute(std::raw::Slice {
-                data: (*ptr).ob_item.as_ptr(),
-                len: self.len()
-            })
+            let ptr = self.0.as_ptr() as *mut ffi::PyTupleObject;
+            PyObject::borrow_from_owned_ptr_slice(
+                slice::from_raw_parts(
+                    (*ptr).ob_item.as_ptr(),
+                    ffi::PyTuple_GET_SIZE(self.0.as_ptr()) as usize
+                ))
         }
     }
-    */
-}
-
-impl <'p> IntoIterator for PyTuple<'p> {
-    type Item = PyObject<'p>;
-    type IntoIter = PyTupleIterator<'p>;
 
     #[inline]
-    fn into_iter(self) -> PyTupleIterator<'p> {
-        PyTupleIterator { index: 0, len: self.len(), tuple: self }
+    pub fn iter<'a, 'p>(&'a self, py: Python<'p>) -> PyTupleIterator<'a, 'p> {
+        PyTupleIterator {
+            py: py,
+            tuple: self,
+            index: 0,
+            end: self.len(py)
+        }
     }
 }
 
-impl <'a, 'p> IntoIterator for &'a PyTuple<'p> {
-    type Item = PyObject<'p>;
-    type IntoIter = PyTupleIterator<'p>;
+impl ::std::ops::Index<usize> for PyTuple {
+    type Output = PyObject;
 
     #[inline]
-    fn into_iter(self) -> PyTupleIterator<'p> {
-        PyTupleIterator { index: 0, len: self.len(), tuple: self.clone() }
+    fn index(&self, index: usize) -> &PyObject {
+        &self.as_slice()[index]
     }
 }
 
-/// Used by `impl IntoIterator for &PyTuple`.
-pub struct PyTupleIterator<'p> {
-    tuple: PyTuple<'p>,
+impl <'a> IntoIterator for &'a PyTuple {
+    type Item = &'a PyObject;
+    type IntoIter = slice::Iter<'a, PyObject>;
+
+    #[inline]
+    fn into_iter(self) -> Self::IntoIter {
+        self.as_slice().iter()
+    }
+}
+
+/// Used by `PyTuple::iter()`.
+pub struct PyTupleIterator<'a, 'p> {
+    py: Python<'p>,
+    tuple: &'a PyTuple,
     index: usize,
-    len: usize
+    end: usize
 }
 
-impl <'p> Iterator for PyTupleIterator<'p> {
-    type Item = PyObject<'p>;
+impl <'a, 'p> Iterator for PyTupleIterator<'a, 'p> {
+    type Item = PyObject;
 
     #[inline]
-    fn next(&mut self) -> Option<PyObject<'p>> {
-        if self.index < self.len {
-            let item = self.tuple.get_item(self.index);
+    fn next(&mut self) -> Option<PyObject> {
+        if self.index < self.end {
+            let item = self.tuple.get_item(self.py, self.index);
             self.index += 1;
             Some(item)
         } else {
@@ -128,43 +140,42 @@ impl <'p> Iterator for PyTupleIterator<'p> {
 
     #[inline]
     fn size_hint(&self) -> (usize, Option<usize>) {
-        (self.len, Some(self.len))
+        (self.len(), Some(self.len()))
     }
 }
 
-impl <'p> ExactSizeIterator for PyTupleIterator<'p> {
+impl <'a, 'p> ExactSizeIterator for PyTupleIterator<'a, 'p> {
     #[inline]
     fn len(&self) -> usize {
-        return self.len;
+        self.end - self.index
     }
 }
 
-fn wrong_tuple_length<'p>(t: &PyTuple<'p>, expected_length: usize) -> PyErr<'p> {
-    let py = t.python();
-    let msg = format!("Expected tuple of length {}, but got tuple of length {}.", expected_length, t.len());
+fn wrong_tuple_length(py: Python, t: &PyTuple, expected_length: usize) -> PyErr {
+    let msg = format!("Expected tuple of length {}, but got tuple of length {}.", expected_length, t.len(py));
     PyErr::new_lazy_init(py.get_type::<exc::ValueError>(), Some(msg.to_py_object(py).into_object()))
 }
 
 macro_rules! id (($a:expr) => ($a));
 
 macro_rules! tuple_conversion ({$length:expr,$(($refN:ident, $n:tt, $T:ident)),+} => (
-    impl <'p, $($T: ToPyObject<'p>),+> ToPyObject<'p> for ($($T,)+) {
-        type ObjectType = PyTuple<'p>;
+    impl <'p, $($T: ToPyObject),+> ToPyObject for ($($T,)+) {
+        type ObjectType = PyTuple;
 
-        fn to_py_object(&self, py: Python<'p>) -> PyTuple<'p> {
+        fn to_py_object(&self, py: Python) -> PyTuple {
             PyTuple::new(py, &[
                 $(id!(self.$n.to_py_object(py)).into_object(),)+
             ])
         }
 
-        fn into_py_object(self, py: Python<'p>) -> PyTuple<'p> {
+        fn into_py_object(self, py: Python) -> PyTuple {
             PyTuple::new(py, &[
                 $(id!(self.$n.into_py_object(py)).into_object(),)+
             ])
         }
     }
 
-    /* TODO: reimplement this without slice matching
+    /*TODO: reimplement this without slice matching
     impl <'p, 's, $($T: FromPyObject<'p, 's>),+> FromPyObject<'p, 's> for ($($T,)+) {
         fn from_py_object(s : &'s PyObject<'p>) -> PyResult<'p, ($($T,)+)> {
             let t = try!(s.cast_as::<PyTuple>());
@@ -175,8 +186,7 @@ macro_rules! tuple_conversion ({$length:expr,$(($refN:ident, $n:tt, $T:ident)),+
                 _ => Err(wrong_tuple_length(t, 2))
             }
         }
-    }
-    */
+    }*/
 ));
 
 tuple_conversion!(1, (ref0, 0, A));
@@ -204,27 +214,44 @@ tuple_conversion!(9, (ref0, 0, A), (ref1, 1, B), (ref2, 2, C), (ref3, 3, D),
 /// let gil_guard = cpython::Python::acquire_gil();
 /// let py = gil_guard.python();
 /// let os = py.import("os").unwrap();
-/// let pid = os.call("get_pid", cpython::NoArgs, None);
+/// let pid = os.call(py, "get_pid", cpython::NoArgs, None);
 /// ```
+#[derive(Copy, Clone, Debug)]
 pub struct NoArgs;
 
 /// Converts `NoArgs` to an empty Python tuple.
-impl <'p> ToPyObject<'p> for NoArgs {
-    type ObjectType = PyTuple<'p>;
+impl ToPyObject for NoArgs {
+    type ObjectType = PyTuple;
 
-    fn to_py_object(&self, py: Python<'p>) -> PyTuple<'p> {
+    fn to_py_object(&self, py: Python) -> PyTuple {
         PyTuple::empty(py)
     }
 }
 
 /// Returns `Ok(NoArgs)` if the input is an empty Python tuple.
 /// Otherwise, returns an error.
-extract!(obj to NoArgs => {
-    let t = try!(obj.cast_as::<PyTuple>());
-    if t.len() == 0 {
+extract!(obj to NoArgs; py => {
+    let t = try!(obj.cast_as::<PyTuple>(py));
+    if t.len(py) == 0 {
         Ok(NoArgs)
     } else {
-        Err(wrong_tuple_length(t, 0))
+        Err(wrong_tuple_length(py, t, 0))
     }
 });
 
+
+
+#[cfg(test)]
+mod test {
+    use python::{Python, PythonObject};
+    use conversion::ToPyObject;
+
+    #[test]
+    fn test_len() {
+        let gil = Python::acquire_gil();
+        let py = gil.python();
+        let tuple = (1, 2, 3).to_py_object(py);
+        assert_eq!(3, tuple.len(py));
+    }
+}
+
