@@ -74,7 +74,7 @@ pub fn prepare_freethreaded_python() {
     });
 }
 
-/// RAII type that represents an acquired GIL.
+/// RAII type that represents the Global Interpreter Lock acquisition.
 ///
 /// # Example
 /// ```
@@ -153,7 +153,7 @@ unsafe impl<T: Send> Sync for GILProtected<T> { }
 impl <T> GILProtected<T> {
     /// Creates a new instance of `GILProtected`.
     #[inline]
-    pub fn new(data: T) -> GILProtected<T> {
+    pub const fn new(data: T) -> GILProtected<T> {
         GILProtected { data: data }
     }
 
